use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Document::Config;

use Clarity -self;

field templatedir => sub { die "TemplateDir not configured\n" };
field fragmentdir => sub { die "FragmentDir not configured\n" };
field contentdir => sub { die "ContentDir not configured\n" };

sub set_templatedir {
	my $dir = $_[0];
	die "Missing argument to TemplateDir\n" unless defined $dir;
	stat $dir;
	warn "TemplateDir $dir doesn't exist\n" unless -e _;
	warn "TemplateDir $dir is not a directory\n" unless -d _;
	$self->templatedir($dir);
}

sub set_fragmentdir {
	my $dir = $_[0];
	die "Missing argument to FragmentDir\n" unless defined $dir;
	stat $dir;
	warn "FragmentDir $dir doesn't exist\n" unless -e _;
	warn "FragmentDir $dir is not a directory\n" unless -d _;
	$self->fragmentdir($dir);
}

sub set_contentdir {
	my $dir = $_[0];
	die "Missing argument to ContentDir\n" unless defined $dir;
	stat $dir;
	warn "ContentDir $dir doesn't exist\n" unless -e _;
	warn "ContentDir $dir is not a directory\n" unless -d _;
	$self->contentdir($dir);
}

package Xyzzy::Document;

use XML::LibXML;
use XML::LibXML::XPathContext;

use Clarity -self;
use Xyzzy::Hook -mixin;

field xml => sub { new XML::LibXML::Document("1.0", "UTF-8") };
field cfg => sub { shift->req->cfg };
field req;

sub _gethash() {
	return $_[0]->{$_[1]}
}

sub getElementById {
	my $id = shift;

#	our $expr ||= new XML::LibXML::XPathExpression('//*[@id=$id]');
	my $expr = '//*[@id=$id]';

	our $ctx ||= new XML::LibXML::XPathContext;

	our $prev;
	$ctx->registerVarLookupFunc(\&_gethash, {id => $id})
		unless defined $prev && $prev eq $id;

	$prev = $id;

	my $nodes = $ctx->find($expr, $self->xml);

	confess "Document ID '$id' not unique"
		if @$nodes > 1;

	return $nodes->[0]
}

sub xmlParser {
	our $xmlparser ||= new XML::LibXML;
	$xmlparser->validation(0);
	$xmlparser->load_ext_dtd(0);
	$xmlparser->expand_entities(0);
	$xmlparser->pedantic_parser(1);
	return $xmlparser;
}

sub serialize {
	return new Xyzzy::Response(mimetype => 'application/xml', content => $self->toString);
}

sub finalize {
#	$self->xml->validate;
	my $res = $self->serialize;
	delete $self->{xml};
	return $res;
}

sub response {
	$self->hook_call('build');
	return $self->finalize;
}

sub DESTROY() {} # don't try to autoload this

sub AUTOLOAD {
	my $sub = our $AUTOLOAD;
	my $off = rindex($sub, '::');
	confess "no package name in '$sub'"
		if $off == -1;
	my $pkg = substr($sub, 0, $off + 2, '');
	my $xml = $self->xml;
	unless($xml->can($sub)) {
		substr($pkg, -2) = '';
		confess "Can't locate object method \"$sub\" via package \"$pkg\"";
	}
	my $code = "sub $sub { my \$self = shift; return \$self->xml->$sub(\@_) }";
	my $err = do { local $@; eval $code; $@ };
	confess $err if $err;
	return $xml->$sub(@_);
}
