use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Request;

use Clarity -self;

use Scalar::Util qw(reftype);

use Xyzzy::Response;
use Xyzzy::Status;

field ctx;
field cfg => sub { shift->ctx->cfg };
field cgi => sub { shift->ctx->cgi };

our @EXPORT_BASE = qw(param);

sub secure {
	return lc($self->cgi->https) eq 'on';
}

sub msie {
	return $self->{msie} if exists $self->{msie};
	my $ua = $self->cgi->user_agent;
	return $self->{msie} = defined $ua && index($ua, 'MSIE') != -1;
}

sub error {
    return $self->cfg->error(@_);
}

sub param(*@) {
	my $name = shift;
	return $name->cgi->param(@_)
		if ref $name;
	my $package = caller;
	my $init;
	my $initcode = 'return $self->{$name} = $self->cgi->param($name)';
	if(@_) {
		$init = shift;
		if(ref $init) {
			if(reftype $init eq 'CODE') {
				$initcode = '
					undef $self->{$name};
					local $_ = $self->cgi->param($name);
					$init->($self, $name, $package, $_);
					return $self->{$name} = $_';
					#	=> sub { die unless defined }
					#	=> sub { die unless $_ }
					#	=> sub { die if /\W/ }
					#	=> sub { $_ ||= "none" }
					#	=> sub { $_ //= "none" }
					#	=> sub { $_ &&= utf8_decode($_) }
					#	=> sub { $_ = lc }
			} else {
				$init = Clarity::Factory::factory($init);
				$initcode .= ' // $init->()';
			}
		} else {
			$initcode .= ' // $init';
		}
	}
	my $code = "
		package $package;
		sub $name {
			my \$self = shift;
			return \$self->{\$name} if exists \$self->{\$name};
			$initcode;
		}
	";
	my $err = do { local $@; eval $code; $@ };
	confess $err if $err;
}

sub DESTROY {} # don't try to autoload this

sub AUTOLOAD {
	my $sub = our $AUTOLOAD;
	my $off = rindex($sub, '::');
	confess "no package name in '$sub'"
		if $off == -1;
	my $pkg = substr($sub, 0, $off + 2, '');
	my $cgi = $self->cgi;
	unless($cgi->can($sub)) {
		substr($pkg, -2) = '';
		confess "Can't locate object method \"$sub\" via package \"$pkg\"";
	}
	my $code = "sub $sub { my \$self = shift; return \$self->cgi->$sub(\@_) }";
	my $err = do { local $@; eval $code; $@ };
	confess $err if $err;
	return $cgi->$sub(@_);
}
