use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Response;

use Clarity -self;

use CGI::Util;

use Xyzzy::Header;
use Xyzzy::Status;

field headers => {
		'Pragma' => [new Xyzzy::Header('no-cache')],
		'Cache-Control' => [
				new Xyzzy::Header('no-cache'),
				new Xyzzy::Header('must-revalidate')
			],
		'Expires' => [new Xyzzy::Header('Thu, 01 Jan 1970 00:00:00 GMT')],
		'Content-Type' => [new Xyzzy::Header('text/html', charset => 'UTF-8')],
	};

field content;

sub canonicalheader() {
	my $key = $_[0];
	confess "Invalid header name '$key'"
		unless Xyzzy::Header::istoken($key);
	$key =~ s/([a-z]+)/ucfirst(lc($1))/gei;
	return $key;
}

sub addheader {
	my $key = canonicalheader(shift);

	my $headers = $self->headers;
	my $header = $headers->{$key};
	$headers->{$key} = $header = []
		unless defined $header;

	push @$header, shift
		while ref $_[0];

	push @$header, new Xyzzy::Header(@_)
		if @_;
}

sub setheader {
	my $key = canonicalheader(shift);

	if(@_ == 1 && !defined $_[0]) {
		delete $self->headers->{$key};
		return;
	}

	my @header;

	push @header, shift
		while ref $_[0];

	push @header, new Xyzzy::Header(@_)
		if @_;

	$self->headers->{$key} = \@header;
}

sub setcookie {
	my $name = shift;
	my $value = CGI::Util::escape(shift);
	my %defaults = (
			Path => $self->url(-absolute => 1),
			Version => 1,
		);
	undef $defaults{secure}
		if $self->secure;
	my @args;
	while(@_) {
		my $key = canonicalheader(shift);
		my $val = shift;
		delete $defaults{$key};
		if($key eq 'Secure') {
			next if defined $val && !$val;
			undef $val;
		}
		push @args, $key, $val;
	}
	while(my ($key, $val) = each(%defaults)) {
		push @args, canonicalheader($key), $val;
	}

	$self->addheader('Set-Cookie', $name => $value, @args);
}

sub status {
	if(@_) {
		return $self->{status} = $_[0] if ref $_[0];
		return $self->{status} = new Xyzzy::Status(@_);
	}
	return $self->{status} if exists $self->{status};
	return $self->{status} = new Xyzzy::Status(200);
}

sub mimetype {
	$self->setheader('Content-Type', @_);
}
